package sun.util.locale.provider;

import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.io.BufferedInputStream;
import java.util.MissingResourceException;
import java.io.IOException;
import sun.text.SupplementaryCharacterData;
import sun.text.CompactByteArray;

class BreakDictionary
{
    private static int supportedVersion;
    private CompactByteArray columnMap;
    private SupplementaryCharacterData supplementaryCharColumnMap;
    private int numCols;
    private int numColGroups;
    private short[] table;
    private short[] rowIndex;
    private int[] rowIndexFlags;
    private short[] rowIndexFlagsIndex;
    private byte[] rowIndexShifts;
    
    BreakDictionary(String s) throws IOException, MissingResourceException {
        this.columnMap = null;
        this.supplementaryCharColumnMap = null;
        this.table = null;
        this.rowIndex = null;
        this.rowIndexFlags = null;
        this.rowIndexFlagsIndex = null;
        this.rowIndexShifts = null;
        this.readDictionaryFile(s);
    }
    
    private void readDictionaryFile(final String className) throws IOException, MissingResourceException {
        BufferedInputStream bufferedInputStream;
        try {
            bufferedInputStream = (BufferedInputStream)AccessController.doPrivileged(new PrivilegedExceptionAction<BufferedInputStream>() {
                @Override
                public BufferedInputStream run() throws Exception {
                    return new BufferedInputStream(this.getClass().getResourceAsStream("/sun/text/resources/" + className));
                }
            });
        }
        catch (PrivilegedActionException cause) {
            throw new InternalError(cause.toString(), cause);
        }
        byte[] b = new byte[8];
        if (bufferedInputStream.read(b) != 8) {
            throw new MissingResourceException("Wrong data length", className, "");
        }
        int int1 = RuleBasedBreakIterator.getInt(b, 0);
        if (int1 != BreakDictionary.supportedVersion) {
            throw new MissingResourceException("Dictionary version(" + int1 + ") is unsupported", className, "");
        }
        int int2 = RuleBasedBreakIterator.getInt(b, 4);
        byte[] b2 = new byte[int2];
        if (bufferedInputStream.read(b2) != int2) {
            throw new MissingResourceException("Wrong data length", className, "");
        }
        bufferedInputStream.close();
        int n = 0;
        int int3 = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        short[] indexArray = new short[int3];
        for (int i = 0; i < int3; ++i, n += 2) {
            indexArray[i] = RuleBasedBreakIterator.getShort(b2, n);
        }
        int int4 = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        byte[] newValues = new byte[int4];
        for (int j = 0; j < int4; ++j, ++n) {
            newValues[j] = b2[n];
        }
        this.columnMap = new CompactByteArray(indexArray, newValues);
        this.numCols = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        this.numColGroups = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        int int5 = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        this.rowIndex = new short[int5];
        for (int k = 0; k < int5; ++k, n += 2) {
            this.rowIndex[k] = RuleBasedBreakIterator.getShort(b2, n);
        }
        int int6 = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        this.rowIndexFlagsIndex = new short[int6];
        for (int l = 0; l < int6; ++l, n += 2) {
            this.rowIndexFlagsIndex[l] = RuleBasedBreakIterator.getShort(b2, n);
        }
        int int7 = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        this.rowIndexFlags = new int[int7];
        for (int n2 = 0; n2 < int7; ++n2, n += 4) {
            this.rowIndexFlags[n2] = RuleBasedBreakIterator.getInt(b2, n);
        }
        int int8 = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        this.rowIndexShifts = new byte[int8];
        for (int n3 = 0; n3 < int8; ++n3, ++n) {
            this.rowIndexShifts[n3] = b2[n];
        }
        int int9 = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        this.table = new short[int9];
        for (int n4 = 0; n4 < int9; ++n4, n += 2) {
            this.table[n4] = RuleBasedBreakIterator.getShort(b2, n);
        }
        int int10 = RuleBasedBreakIterator.getInt(b2, n);
        n += 4;
        int[] table = new int[int10];
        for (int n5 = 0; n5 < int10; ++n5, n += 4) {
            table[n5] = RuleBasedBreakIterator.getInt(b2, n);
        }
        this.supplementaryCharColumnMap = new SupplementaryCharacterData(table);
        return;
    }
    
    public final short getNextStateFromCharacter(int n, int index) {
        int n2;
        if (index < 65536) {
            n2 = this.columnMap.elementAt((char)index);
        }
        else {
            n2 = this.supplementaryCharColumnMap.getValue(index);
        }
        return this.getNextState(n, n2);
    }
    
    public final short getNextState(int n, int n2) {
        if (this.cellIsPopulated(n, n2)) {
            return this.internalAt((int)this.rowIndex[n], n2 + this.rowIndexShifts[n]);
        }
        return 0;
    }
    
    private boolean cellIsPopulated(int n, int n2) {
        if (this.rowIndexFlagsIndex[n] < 0) {
            return n2 == -this.rowIndexFlagsIndex[n];
        }
        return (this.rowIndexFlags[this.rowIndexFlagsIndex[n] + (n2 >> 5)] & 1 << (n2 & 0x1F)) != 0x0;
    }
    
    private short internalAt(int n, int n2) {
        return this.table[n * this.numCols + n2];
    }
    
    static {
        BreakDictionary.supportedVersion = 1;
    }
}
